package com.pms.ocp.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.pms.ocp.mapper.ModelCheckMapper;
import com.pms.ocp.model.entity.ModelPropInfo;
import com.pms.ocp.model.entity.PropInfo;
import com.pms.ocp.service.ModelCheckRuleService;
import org.apache.commons.collections4.list.TreeList;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.LinkedList;
import java.util.List;

/**
 * @author huxiuwu
 * @version 1.0
 * @date 2022/4/11 17:17
 */
@Service
public class ModelCheckRuleServiceImpl implements ModelCheckRuleService {

    @Resource
    private ModelCheckMapper modelCheckMapper;

    /**
     * @param modelCode       模型编码
     * @param requirementCode 需求编码
     * @return List<String>
     * @author huxiuwu
     * @date 2022/4/11
     **/
    @Override
    public List<String> checkRule(String modelCode, String requirementCode) {
        List<ModelPropInfo> ownerProp = modelCheckMapper.selectModelOwnerProp(modelCode);
        List<String> result = checkModelRule(ownerProp);
        return result;
    }

    /**
     * @param ownerProp 自己的属性
     * @return List<String>
     * @author huxiuwu
     * @date 2022/4/11
     **/
    private List<String> checkModelRule(List<ModelPropInfo> ownerProp) {
        ModelPropInfo modelPropInfo = ownerProp.get(0);
        //校验
        List<String> checkResult = checkRuleOne(modelPropInfo);
        return checkResult;
    }

    private List<String> checkRuleOne(ModelPropInfo modelPropInfo) {
        List<String> temp = new TreeList<>();
        final String regular = ("([a-z]+)|([A-Z][a-z]*)|([A-Z]+)");
        String modelTable = modelPropInfo.getModelTable();
        String tableName = modelTable == null ? "" : modelTable;
        if (!tableName.matches(regular) || checkRuleOther(tableName)) {
            temp.add("表名要小写，下划线风格，采用四段式，不超过30个字符串");
        }
        temp = checkRuleTwo(modelPropInfo, temp);
        return temp;
    }

    private List<String> checkRuleTwo(ModelPropInfo modelPropInfo, List<String> temp) {
        List<PropInfo> propList = modelPropInfo.getPropList();
        if (CollectionUtil.isEmpty(propList)) {
            temp.add("属性必须有注释");
            return temp;
        }
        List<Boolean> isHasPk = new LinkedList<>();
        boolean isPk;
        for (PropInfo propInfo : propList) {
            if (propInfo.getColumnComments() == null) {
                temp.add("属性必须有注释");
            }
            isPk = propInfo.getIsPk()==1;
            if (isPk){
                isHasPk.add(isPk);
            }
        }
        if (CollectionUtil.isEmpty(isHasPk)){
            temp.add("未有主键");
        }
        return temp;
    }

    private boolean checkRuleOther(String tableName) {
        int length = tableName.split("_").length;
        if (length <= 0||tableName.length() >= 30) {
            return true;
        }
        return false;
    }

}
